# an LP solver for Joho-Kiso 
# 2016, Yoshinori Hayakawa   
# CITE, Tohoku university 
# 2019-02-21 Python 3 version
# coding: utf-8

import sys

class SimplexMethod:
    def __init__(self,n,m):
        self.N=n
        self.M=m
        self.A=[[0 for i in range(n+m+1)] for j in range(m+1)]
        self.B=[0]*(m+1)
        return

    def init_coef(self,alpha,b,c):
        for i in range(self.M):
            self.B[i] = b[i]
            for j in range(self.N + self.M + 1):
                if j<self.N:
                    self.A[i][j] = alpha[i][j]
                else:
                    if j == self.N + i:
                        self.A[i][j]= 1.0
                    else:
                        self.A[i][j]= 0.0

        self.B[self.M]= 0.0
        for j in range(self.N + self.M + 1):
            if j<self.N:
                self.A[self.M][j] = -c[j]
            else:
                if j == self.N+self.M:
                    self.A[self.M][j]= -1 
                else:
                    self.A[self.M][j]=0.0

    # An implementation of Algorithm 13 
    def pivot(self):
        found_negative_gamma=False

        for j in range(self.N+self.M):
            if self.A[self.M][j] < 0.0:
                s= j
                found_negative_gamma=True
                break
            
        if  not found_negative_gamma:
            return "done" # FOUND SOLUTION

        found_positive_alpha=False
        for i in range(self.M):
            if self.A[i][s] > 0.0:
                found_positive_alpha=True

        if not found_positive_alpha:
            return "unbounded" # UNBOUNDED

        v = sys.float_info.max
        for i in range(self.M):
            if self.A[i][s]>0:
                if self.B[i]/self.A[i][s] < v:
                    v = self.B[i]/self.A[i][s]
                    r = i
        a_rs=self.A[r][s]
        for j in range(self.N + self.M):
            self.A[r][j] = self.A[r][j]/a_rs
        self.B[r] = self.B[r]/a_rs

        for i in range(self.M+1):
            if i!=r:
                a_is = self.A[i][s]
                for j in range(self.N + self.M):
                    self.A[i][j] = self.A[i][j] - self.A[r][j]*a_is
                self.B[i] = self.B[i] - self.B[r]*a_is 

        return "canonical form"

    # An implementation of Algorithm 14 
    def simplex(self):
        res = self.pivot()
        if res == "unbounded":
            print("unbounded")
            return -1
        elif res=="done":
            return 1 
        else:
            return self.simplex()


    def dump_matrix(self):
        for i in range(self.M+1):
            for j in range (self.N+self.M+1):
                print(self.A[i][j]," ",end="") 
            print("  ",self.B[i])

    def get_opt_param(self,j):
        n_unity=0
        n_nonzero=0 ;
        for i in range(self.M):
            if abs(self.A[i][j]-1.0)<1.0e-6 :
                n_unity=n_unity+1
                s=i
            elif abs(self.A[i][j])>1.0e-6:
                n_nonzero=n_nonzero+1

        if n_unity==1 and n_nonzero==0:
            return self.B[s]
        else:
            return 0

    def get_opt_value(self):
        return self.B[self.M]

    def print_solution(self):
        for j in range(self.N):
            val = self.get_opt_param(j) 
            print("x[",j,"]= ",val,sep="")
        print("z=",self.get_opt_value())
